package net.rumati.ezsql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class EZSQL
{
    private EZSQL(){}

    public static int executeUpdateSQL(Connection conn, String sql) throws SQLException
    {
        return executeUpdateSQL(conn, sql, new ParameterSetter[0]);
    }

    public static int executeUpdateSQL(Connection conn, String sql, ParameterSetter parameter) throws SQLException
    {
        return executeUpdateSQL(conn, sql, new ParameterSetter[]{parameter});
    }

    public static int executeUpdateSQL(Connection conn, String sql, ParameterSetter[] parameters) throws SQLException
    {
        return executeUpdateSQL(conn, sql, Arrays.asList(parameters));
    }

    public static int executeUpdateSQL(Connection conn, String sql, final List<ParameterSetter> parameters)
        throws SQLException
    {
        return withPreparedStatement(conn.prepareStatement(sql), new PreparedStatementHandler<Integer>() {

            @Override
            public Integer handlePreparedStatement(PreparedStatement pst)
                    throws SQLException
            {
                int size = parameters.size();
                for (int i = 0; i < size; i++){
                    parameters.get(i).setParameter(pst, i + 1);
                }
                return pst.executeUpdate();
            }
        });
    }

    public static <T> T withPreparedStatement(PreparedStatement pth, PreparedStatementHandler<T> handler) throws SQLException
    {
        try {
            return handler.handlePreparedStatement(pth);
        }finally{
            try {
                pth.close();
            }catch (SQLException ignore){}
        }
    }

    public static <T> T withResultSet(Connection conn, String sql, final ResultSetHandler<T> handler) throws SQLException
    {
        return withResultSet(conn, sql, new ParameterSetter[]{}, handler);
    }

    public static <T> T withResultSet(Connection conn, String sql, ParameterSetter parameter, final ResultSetHandler<T> handler) throws SQLException
    {
        return withResultSet(conn, sql, new ParameterSetter[]{parameter}, handler);
    }

    public static <T> T withResultSet(Connection conn, String sql, ParameterSetter[] parameters, ResultSetHandler<T> handler) throws SQLException
    {
        return withResultSet(conn, sql, Arrays.asList(parameters), handler);
    }

    public static <T> T withResultSet(Connection conn, String sql, final List<ParameterSetter> parameters, final ResultSetHandler<T> handler) throws SQLException
    {
        return withPreparedStatement(conn.prepareStatement(sql), new PreparedStatementHandler<T>() {
            @Override
            public T handlePreparedStatement(PreparedStatement pst)
                    throws SQLException
            {
                int size = parameters.size();
                for (int i = 0; i < size; i++){
                    parameters.get(i).setParameter(pst, i + 1);
                }
                ResultSet rs = pst.executeQuery();
                try {
                    return handler.handleResultSet(rs);
                }finally{
                    try {
                        rs.close();
                    }catch (SQLException ignore){}
                }
            }
        });
    }

    public static <T> List<T> getAllResults(Connection conn, String sql, RowHandler<T> rowHandler) throws SQLException
    {
        return getAllResults(conn, sql, new LinkedList<ParameterSetter>(), rowHandler);
    }

    public static <T> List<T> getAllResults(Connection conn, String sql, ParameterSetter parameter, RowHandler<T> rowHandler) throws SQLException
    {
        return getAllResults(conn, sql, new ParameterSetter[]{parameter}, rowHandler);
    }

    public static <T> List<T> getAllResults(Connection conn, String sql, ParameterSetter[] parameters, RowHandler<T> rowHandler) throws SQLException
    {
        return getAllResults(conn, sql, Arrays.asList(parameters), rowHandler);
    }

    public static <T> List<T> getAllResults(Connection conn, String sql, List<ParameterSetter> parameters, RowHandler<T> rowHandler) throws SQLException
    {
        return withResultSet(conn, sql, parameters, ResultSetHandler.getAllRows(rowHandler));
    }

    public static <T> T getFirstResult(Connection conn, String sql, RowHandler<T> rowHandler) throws SQLException
    {
        return getFirstResult(conn, sql, new LinkedList<ParameterSetter>(), rowHandler);
    }

    public static <T> T getFirstResult(Connection conn, String sql, ParameterSetter parameter, RowHandler<T> rowHandler) throws SQLException
    {
        return getFirstResult(conn, sql, new ParameterSetter[]{parameter}, rowHandler);
    }

    public static <T> T getFirstResult(Connection conn, String sql, ParameterSetter[] parameters, RowHandler<T> rowHandler) throws SQLException
    {
        return getFirstResult(conn, sql, Arrays.asList(parameters), rowHandler);
    }

    public static <T> T getFirstResult(Connection conn, String sql, List<ParameterSetter> parameters, RowHandler<T> rowHandler) throws SQLException
    {
        return withResultSet(conn, sql, parameters, ResultSetHandler.getFirstRow(rowHandler));
    }

    public static <T> T getOneColumn(Connection conn, String sql, final ColumnHandler<T> columnHandler) throws SQLException
    {
        return getOneColumn(conn, sql, new ParameterSetter[]{}, columnHandler);
    }

    public static <T> T getOneColumn(Connection conn, String sql, ParameterSetter parameter, final ColumnHandler<T> columnHandler) throws SQLException
    {
        return getOneColumn(conn, sql, new ParameterSetter[]{parameter}, columnHandler);
    }

    public static <T> T getOneColumn(Connection conn, String sql, ParameterSetter[] parameters, final ColumnHandler<T> columnHandler) throws SQLException
    {
        return getOneColumn(conn, sql, Arrays.asList(parameters), columnHandler);
    }

    public static <T> T getOneColumn(Connection conn, String sql, List<ParameterSetter> parameters, final ColumnHandler<T> columnHandler) throws SQLException
    {
        return getFirstResult(conn, sql, parameters, new RowHandler<T>() {
            @Override
            public T handleRow(Row r)
                    throws SQLException
            {
                return columnHandler.handleColumn(Column.fromIndex(r, 1));
            }
        });
    }

    public static String getOneString(Connection conn, String sql) throws SQLException
    {
        return getOneString(conn, sql, new ParameterSetter[]{});
    }

    public static String getOneString(Connection conn, String sql, ParameterSetter parameter) throws SQLException
    {
        return getOneString(conn, sql, new ParameterSetter[]{parameter});
    }

    public static String getOneString(Connection conn, String sql, ParameterSetter[] parameters) throws SQLException
    {
        return getOneString(conn, sql, Arrays.asList(parameters));
    }

    public static String getOneString(Connection conn, String sql, List<ParameterSetter> parameters) throws SQLException
    {
        return getOneColumn(conn, sql, parameters, new ColumnHandler<String>() {

            @Override
            public String handleColumn(Column col)
                    throws SQLException
            {
                return col.getString();
            }
        });
    }

    public static int getOneInt(Connection conn, String sql) throws SQLException
    {
        return getOneInt(conn, sql, new ParameterSetter[]{});
    }

    public static int getOneInt(Connection conn, String sql, ParameterSetter parameter) throws SQLException
    {
        return getOneInt(conn, sql, new ParameterSetter[]{parameter});
    }

    public static int getOneInt(Connection conn, String sql, ParameterSetter[] parameters) throws SQLException
    {
        return getOneInt(conn, sql, Arrays.asList(parameters));
    }

    public static int getOneInt(Connection conn, String sql, List<ParameterSetter> parameters) throws SQLException
    {
        return getOneColumn(conn, sql, parameters, new ColumnHandler<Integer>() {

            @Override
            public Integer handleColumn(Column col)
                    throws SQLException
            {
                return col.getInt();
            }
        });
    }
}
