package net.rumati.ezsql;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Calendar;

/**
 * Holds a JDBC parameter which canbe used at a later stage to update a {@link java.sql.PreparedStatement}
 * by populating a parameter. {@link java.sql.PreparedStatement}s require parameters to be set using the correct
 * method, but keeping that information (ie. parameter type) is awkward. This class alloas you to create a
 * ParameterSetter as an early stage, while parameter type is still known, by calling a static method named
 * similarly to the {@link PreparedStatement} <code>set*</code> methods. The resultant ParameterSetter can be used
 * later to update the {@link PreparedStatement} using the {@link #setParameter(java.sql.PreparedStatement, int) }
 * method.
 *
 * @author jesse
 */
public abstract class ParameterSetter
{
    private static enum Type
    {
        BOOLEAN(Types.BOOLEAN),
        BYTE(Types.TINYINT),
        DOUBLE(Types.DOUBLE),
        FLOAT(Types.FLOAT),
        INT(Types.INTEGER),
        LONG(Types.BIGINT),
        SHORT(Types.SMALLINT);
        
        private final int jdbcSqlType;

        private Type(int jdbcSqlType)
        {
            this.jdbcSqlType = jdbcSqlType;
        }

        public int getJdbcSqlType()
        {
            return jdbcSqlType;
        }
    }

    private ParameterSetter(){}

    public abstract void setParameter(PreparedStatement pth, int parameterNumber)
        throws SQLException;
    
    private static ParameterSetter setNull(final Type t)
    {
        return new ParameterSetter() {

            @Override
            public void setParameter(PreparedStatement pth, int parameterNumber)
                    throws SQLException
            {
                pth.setNull(parameterNumber, t.getJdbcSqlType());
            }
        };
    }

    public static ParameterSetter setNullableBoolean(final Boolean value)
    {
        if (value == null){
            return setNull(Type.BOOLEAN);
        }
        return setBoolean(value);
    }

    public static ParameterSetter setNullableByte(Byte value)
    {
        if (value == null){
            return setNull(Type.BYTE);
        }
        return setByte(value);
    }

    public static ParameterSetter setNullableDouble(Double value)
    {
        if (value == null){
            return setNull(Type.DOUBLE);
        }
        return setDouble(value);
    }

    public static ParameterSetter setNullableFloat(Float value)
    {
        if (value == null){
            return setNull(Type.FLOAT);
        }
        return setFloat(value);
    }

    public static ParameterSetter setNullableInt(Integer value)
    {
        if (value == null){
            return setNull(Type.INT);
        }
        return setInt(value);
    }

    public static ParameterSetter setNullableLong(Long value)
    {
        if (value == null){
            return setNull(Type.LONG);
        }
        return setLong(value);
    }

    public static ParameterSetter setNullableShort(Short value)
    {
        if (value == null){
            return setNull(Type.SHORT);
        }
        return setShort(value);
    }

    public static ParameterSetter setArray(final Array array)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setArray(parameterNumber, array);
            }
        };
    }

    public static ParameterSetter setAsciiStream(final InputStream ins)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setAsciiStream(parameterNumber, ins);
            }
        };
    }

    public static ParameterSetter setAsciiStream(final InputStream ins, final int length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setAsciiStream(parameterNumber, ins, length);
            }
        };
    }

    public static ParameterSetter setAsciiStream(final InputStream ins, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setAsciiStream(parameterNumber, ins, length);
            }
        };
    }

    public static ParameterSetter setBigDecimal(final BigDecimal value)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBigDecimal(parameterNumber, value);
            }
        };
    }

    public static ParameterSetter setBinaryStream(final InputStream ins)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBinaryStream(parameterNumber, ins);
            }
        };
    }

    public static ParameterSetter setBinaryStream(final InputStream ins, final int length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBinaryStream(parameterNumber, ins, length);
            }
        };
    }

    public static ParameterSetter setBinaryStream(final InputStream ins, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBinaryStream(parameterNumber, ins, length);
            }
        };
    }

    public static ParameterSetter setBlob(final Blob x)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBlob(parameterNumber, x);
            }
        };
    }

    public static ParameterSetter setBlob(final InputStream in)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBlob(parameterNumber, in);
            }
        };
    }

    public static ParameterSetter setBlob(final InputStream in, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBlob(parameterNumber, in, length);
            }
        };
    }

    public static ParameterSetter setBoolean(final boolean b)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBoolean(parameterNumber, b);
            }
        };
    }

    public static ParameterSetter setByte(final byte b)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setByte(parameterNumber, b);
            }
        };
    }

    public static ParameterSetter setBytes(final byte[] b)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setBytes(parameterNumber, b);
            }
        };
    }

    public static ParameterSetter setCharacterStream(final Reader r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setCharacterStream(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setCharacterStream(final Reader r, final int length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setCharacterStream(parameterNumber, r, length);
            }
        };
    }

    public static ParameterSetter setCharacterStream(final Reader r, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setCharacterStream(parameterNumber, r, length);
            }
        };
    }

    public static ParameterSetter setClob(final Clob c)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setClob(parameterNumber, c);
            }
        };
    }

    public static ParameterSetter setClob(final Reader r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setClob(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setClob(final Reader r, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setClob(parameterNumber, r, length);
            }
        };
    }

    public static ParameterSetter setDate(final Date d)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setDate(parameterNumber, d);
            }
        };
    }

    public static ParameterSetter setDate(final Date d, final Calendar c)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setDate(parameterNumber, d, c);
            }
        };
    }

    public static ParameterSetter setDouble(final double d)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setDouble(parameterNumber, d);
            }
        };
    }

    public static ParameterSetter setFloat(final float f)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setFloat(parameterNumber, f);
            }
        };
    }

    public static ParameterSetter setInt(final int i)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setInt(parameterNumber, i);
            }
        };
    }

    public static ParameterSetter setLong(final long l)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setLong(parameterNumber, l);
            }
        };
    }

    public static ParameterSetter setNCharacterStream(final Reader r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNCharacterStream(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setNCharacterStream(final Reader r, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNCharacterStream(parameterNumber, r, length);
            }
        };
    }

    public static ParameterSetter setNClob(final NClob c)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNClob(parameterNumber, c);
            }
        };
    }

    public static ParameterSetter setNClob(final Reader r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNClob(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setNClob(final Reader r, final long length)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNClob(parameterNumber, r, length);
            }
        };
    }

    public static ParameterSetter setNString(final String value)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNString(parameterNumber, value);
            }
        };
    }

    public static ParameterSetter setNull(final int sqlType)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNull(parameterNumber, sqlType);
            }
        };
    }

    public static ParameterSetter setNull(final int sqlType, final String typeName)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setNull(parameterNumber, sqlType, typeName);
            }
        };
    }

    public static ParameterSetter setObject(final Object o)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setObject(parameterNumber, o);
            }
        };
    }

    public static ParameterSetter setObject(final Object o, final int targetSqlType)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setObject(parameterNumber, o, targetSqlType);
            }
        };
    }

    public static ParameterSetter setObject(final Object o, final int targetSqlType,
            final int scaleOrLength)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setObject(parameterNumber, o, targetSqlType, scaleOrLength);
            }
        };
    }

    public static ParameterSetter setRef(final Ref r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setRef(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setRowId(final RowId r)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setRowId(parameterNumber, r);
            }
        };
    }

    public static ParameterSetter setSQLXML(final SQLXML value)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setSQLXML(parameterNumber, value);
            }
        };
    }

    public static ParameterSetter setShort(final short s)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setShort(parameterNumber, s);
            }
        };
    }

    public static ParameterSetter setString(final String str)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setString(parameterNumber, str);
            }
        };
    }

    public static ParameterSetter setTime(final Time t)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setTime(parameterNumber, t);
            }
        };
    }

    public static ParameterSetter setTime(final Time t, final Calendar cal)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setTime(parameterNumber, t, cal);
            }
        };
    }

    public static ParameterSetter setTimestamp(final Timestamp t)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setTimestamp(parameterNumber, t);
            }
        };
    }

    public static ParameterSetter setTimestamp(final Timestamp t, final Calendar cal)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setTimestamp(parameterNumber, t, cal);
            }
        };
    }

    public static ParameterSetter setURL(final URL u)
    {
        return new ParameterSetter() {
            @Override
            public void setParameter(PreparedStatement psth,
                                                int parameterNumber)
                    throws SQLException
            {
                psth.setURL(parameterNumber, u);
            }
        };
    }
}
